function [V1,V2,r,U,V,stats] = emiCCA(X,Y,Pro_index,flag,P)
% emiCCA: eigenspace mutual-information  canonical correlation analysis. (beta v1)
%   [V1,V2] = emiCCA(X,Y,Pro_index,flag,P) computes the the Mutual-Information
%   Eigen Coefficient (MIEC) for the N-by-P1 and N-by-P2 data matrices X and Y.
%   Pro_index is the probability of MIC coefficients generated from null distribution.
%   It is used to calculate the significance of canonical information coefficients (r).
% Input:
%   X:        Data set X. e.g. N sample size X P1 dimensions.
%   Y:        Data set Y. e.g. N sample size X P2 dimensions.
%   Pro_index:
%             First column is probabilities; second column is MIC value;
%             this is optional.
%   flag:
%         flag = 0 (default) -> not reject non-significant MIC coefficients.
%         flag = 1 -> use uncorrected p-value to reject non-significant MIC coefficients.
%         flag = 2 -> use FDR corrected p-value to reject non-significant MIC coefficients.
%   P:    Threshold of MIC p-value. If flag == 1 or flag == 2, The input P must be required.
%         P > 0. Default is 1.
%
% Caution:  If you want to get the p-value of r, Pro_index is required;
%           If you not want to, it can be ignored. BUT, if there is no Pro_index,
%           last three inputs should be ignored, and all p-values of canonical
%           information coefficients (r) is default to 1.
%           !!!!!!!!!!!
%
%
%   X and Y must have the same number of observations (rows) but can have
%   different numbers of variables (cols).  A and B are P1-by-D and P2-by-D
%   matrices, where D = min(rank(X),rank(Y)).
%   The jth columns of A and B contain the canonical coefficients,the identity matrix.
%
%   If X or Y are less than full rank, emiCCA gives a warning and
%   returns zeros in the rows of A or B corresponding to dependent columns
%   of X or Y.
%
%   [V1,V2,r] = emiCCA(X,Y,...) returns the 1-by-D vector R containing the
%   sample canonical correlations.  The jth element of R is the correlation
%   between the jth columns of U and V (see below).
%
%   [V1,V2,r,U,V] = emiCCA(X,Y,...) returns the canonical variables, also
%   known as scores, in the N-by-D matrices U and V.  U and V are computed
%   as U = X*V1 and V = Y*V2.
%
%   [V1,V2,r,U,V, stats] = emiCCA(X,Y,...) returns the MIC score bewteen canonical
%   variables, and p-value.
%
%   Example:
%
%      load carbig;
%      X = [Displacement Horsepower Weight Acceleration MPG];
%      nans = sum(isnan(X),2) > 0;
%      [A,B,r,U,V] = emiCCA(X(~nans,1:3), X(~nans,4:5));
%
%      plot(U(:,1),V(:,1),'.');
%      xlabel('0.0025*Disp + 0.020*HP - 0.000025*Wgt');
%      ylabel('-0.17*Accel + -0.092*MPG')
%
%   See also PRINCOMP, MANOVA1.
% -----------------------------------
%   Author: Li Dong
%   Li_dong729@163.com
%   Reference:
%   Li Dong, Yangsong Zhang, Rui Zhang, Xingxing Zhang, Diankun Gong, Pedro A.
%      Valdes-Sosa, Peng Xu, Cheng Luo, Dezhong Yao.Characterizing nonlinear relationships 
%      in functional imaging data using eigenspace maximal information canonical correlation analysis (emiCCA).
%      Neuroimage,2015,Accepted.
%   $Date: 2015/1/16
% =========================================================================
if nargin < 2
    error('stats:emiCCA:TooFewInputs','Requires two arguments.');
elseif nargin == 2
    Pro_index = [0 1];
    P = 1;
    flag = 0;
elseif nargin == 3
    P = 1;
    flag = 0;
end
% -------------------------
% get path of emiCCA function
emiCCApath = which('emiCCA.m');
emiCCApath = emiCCApath(1:end-length('emiCCA.m'));
% =========================================================================
% Factor the inputs
% =========================================================================

[n,p1] = size(X);
% center the variables
X = X - repmat(mean(X,1),n,1);
Y = Y - repmat(mean(Y,1),n,1);
% ----------------------
if size(Y,1) ~= n
    error('stats:emiCCA:InputSizeMismatch',...
        'X and Y must have the same number of rows.');
elseif n == 1
    error('stats:emiCCA:NotEnoughData',...
        'X and Y must have more than one row.');
end
p2 = size(Y,2);
% Factor the inputs, and find a full rank set of columns if necessary
[Q1,T11,perm1] = qr(X,0);
rankX = sum(abs(diag(T11)) > eps(abs(T11(1)))*max(n,p1));
if rankX == 0
    error('stats:emiCCA:BadData',...
        'X must contain at least one non-constant column');
elseif rankX < p1
    warning('stats:emiCCA:NotFullRank','X is not full rank.');
    Q1 = Q1(:,1:rankX); T11 = T11(1:rankX,1:rankX);
end

Xa = Q1*T11;

[Q2,T22,perm2] = qr(Y,0);
rankY = sum(abs(diag(T22)) > eps(abs(T22(1)))*max(n,p2));
if rankY == 0
    error('stats:emiCCA:BadData',...
        'Y must contain at least one non-constant column');
elseif rankY < p2
    warning('stats:emiCCA:NotFullRank','Y is not full rank.');
    Q2 = Q2(:,1:rankY); T22 = T22(1:rankY,1:rankY);
end
Ya =  Q2*T22;
% =========================================================================
% claculate MIC matrix
% =========================================================================
M1 = [(1:(size(Xa,2)+size(Ya,2)))' ([Xa,Ya])'];
% ----------------------------
dataname = [emiCCApath,'temp_data\series1.csv'];
csvwrite(dataname,M1);
% Using MINE to calsulate MIC bewteen tiemcouses.
command_1 =['java -jar ',[emiCCApath,'MINE.jar '],dataname,' -allPairs cv=0 exp=0.55 c=5'];
system(command_1);
% ----------------------------
name = [emiCCApath,'temp_data\series1.csv,allpairs,cv=0.0,B=n^0.55,Results.csv'];
if exist(name,'file') == 2
    fid = fopen(name);
    a1 = textscan(fid,'%s');
    results = [];
    % -----------------------------------
    M2 = zeros(size(M1,1),size(M1,1)); % MIC matrix
    P_value = zeros(size(M1,1),size(M1,1)); % p-value matrix;
    for i1 = 11:size(a1{1,1},1)
        results(i1-10,:) = str2num(a1{1,1}{i1,1});
        % ---------------------------------------
        M2(results(i1-10,1),results(i1-10,2)) = results(i1-10,3);
        % ---------------------------------------
        k = sum(Pro_index(:,2) <= results(i1-10,3));
        P_value(results(i1-10,1),results(i1-10,2)) = Pro_index(max(k,1),1);
        % ------
    end
    % ---------------------------------------------------------------------
    % reject non-significant MIC coefficients.
    if flag == 1   % using uncorrected p-value
        W1 = zeros(size(M1,1),size(M1,1));
        P1 = P_value(tril(ones(size(P_value)),-1)==1);
        if any(P1<=P)
            W1(P_value <= P) = 1;
            M2 = M2 .* W1;
        else
            % clc
            warning('stats:emiCCA:NotReject','Non significant MIC coefficients.');
        end
    elseif flag == 2  % using uncorrected p-value
        W1 = zeros(size(M1,1),size(M1,1));
        P1 = P_value(tril(ones(size(P_value)),-1)==1);
        [~,pN] = FDR(P1,P);  % FDR corrected
        if isempty(pN)~=1
            W1(P_value <= pN) = 1;
            M2 = M2 .* W1;
        else
            % clc
            warning('stats:emiCCA:NotReject','Non significant MIC coefficients.');
        end
    end;
    % --------------------------------------
    M3 = M2 + eye(size(M2)) + M2'; % MIC matrix
    % =========================================================================
    % Compute MIEC and canonical variables.
    % For rankX > rankY, it ignores the extra columns in L and rows in D.
    % For rankX < rankY, need to ignore extra columns in M and D explicitly.
    % Normalize A and B to give U and V unit variance.
    % =========================================================================
    d = min(rankX,rankY);
    
    if rankX == d
        A = pinv(M3(1:rankX,1:rankX))*M3(1:rankX,rankX+1:rankX+rankY)*pinv(M3(rankX+1:rankX+rankY,rankX+1:rankX+rankY))*M3(rankX+1:rankX+rankY,1:rankX);
        [V1,D] = eig(A,'nobalance');
        V2 = pinv(M3(rankX+1:rankX+rankY,rankX+1:rankX+rankY))*M3(rankX+1:rankX+rankY,1:rankX) * V1;
    else
        B = pinv(M3(rankX+1:rankX+rankY,rankX+1:rankX+rankY))*M3(rankX+1:rankX+rankY,1:rankX)*pinv(M3(1:rankX,1:rankX))*M3(1:rankX,rankX+1:rankX+rankY);
        [V2,D] = eig(B,'nobalance');
        V1 = pinv(M3(1:rankX,1:rankX))*M3(1:rankX,rankX+1:rankX+rankY)*V2;
    end;
    % --------------------
    % D = real(D);
    d1 = sqrt(diag(D(:,1:d)));
    r = min(max(abs(d1),0),1); % remove roundoff errs
    % --------------------
    % Put coefficients back to their full size and their correct order
    V1(perm1,:) = [V1; zeros(p1-rankX,d)];
    V2(perm2,:) = [V2; zeros(p2-rankY,d)];
    
    % Compute the canonical variates
    if nargout >3
        V1 = real(V1); % get real part
        V2 = real(V2); % get real part
        std1 = std(X * V1);
        std2 = std(Y * V2);
        std1(std1 == 0)=1;
        std2(std2 == 0)=1;
        V1 = V1./repmat(std1,size(V1,1),1); % give U and V unit variance
        V2 = V2./repmat(std2,size(V2,1),1);
        U = X * V1;
        V = Y * V2;
    end
    
    % =========================================================================
    % Compute test statistics for H0k: rk == 0
    % =========================================================================
    if nargout > 5
        % nonparameter statistic
        U = real(U); % get real part
        V = real(V); % get real part
        stats.MIC = zeros(1,length(r));
        stats.P = ones(1,length(r));
        for i2 = 1:length(r)
            u = U(:,i2); v = V(:,i2);
            if sum(u)~=0 && sum(v)~=0
                % claculate MIC matrix
                M = [(1:2)' ([u,v])'];
                csvwrite([emiCCApath,'temp_data\series2.csv'],M);
                % ----------------------------
                % Using MINE to calsulate MIC bewteen tiemcouses.
                command_2 =['java -jar ',[emiCCApath,'MINE.jar '],...
                            [emiCCApath,'temp_data\series2.csv'],' -allPairs cv=0 exp=0.55 c=5'];
                system(command_2);
                % -----------------------------
                name = [emiCCApath,'temp_data\series2.csv,allpairs,cv=0.0,B=n^0.55,Results.csv'];
                fid = fopen(name);
                a1 = textscan(fid,'%s');
                a2 = str2num(a1{1,1}{11,1});
                % U(i) vs V(i) MIC value
                stats.MIC(1,i2) = a2(3);
                % p-value
                if nargin >= 3
                    k = sum(Pro_index(:,2) <= a2(3));
                    stats.P(1,i2) = Pro_index(max(k,1),1);
                end
                % ------------------------------
            end
        end
    end;
else
    V1 = [];
    V2 = [];
    r = [];
    U = [];
    V = [];
    stats = [];
    warning('please check your data or MINE toolbox');
end
% -----------------------
fopen('all'); % List all open files
fclose('all'); % Close all open files
% -----------------------
end

% =========================================================================
% sub function
function [pID,pN] = FDR(p,q)
% FORMAT [pID,pN] = FDR(p,q)
%
% p   - vector of p-values
% q   - False Discovery Rate level
%
% pID - p-value threshold based on independence or positive dependence
% pN  - Nonparametric p-value threshold
%______________________________________________________________________________
% $Id: FDR.m,v 1.1 2009/10/20 09:04:30 nichols Exp $
p = p(isfinite(p));  % Toss NaN's
p = sort(p(:));
V = length(p);
I = (1:V)';

cVID = 1;
cVN = sum(1./(1:V));

pID = p(max(find(p <= I/V*q/cVID)));
pN = p(max(find(p <= I/V*q/cVN)));
end